#===================================================================================================
#Program emulates the functionality of a Lantronix UDS-10 as used by Stuart's TI Internet Browser
#(www.stuartconner.me.uk/ti/ti.htm#internet_web_browser).

#Program tested on a Windows 7 PC, Raspberry Pi 2 Model B and Raspberry Pi 3 Model B.
#Program compatible with the later versions of Python 2 and Python 3.

#See the website referenced above for full details on how to set the name of the serial port and use
#the program.
#===================================================================================================

def read_from_serial_port():

    while True: #Loop until at least one character received from serial port.
            
        serial_text_bytes = ser.read(5000) #Read up to 5000 characters from serial port, or time out. Text read as a bytes literal.
        serial_text = serial_text_bytes.decode('utf-8') #Convert bytes literal to a normal string.

        if len(serial_text) > 0: #At least one character received - exit loop.
            break

    return serial_text

#===================================================================================================

import serial
import socket

#Initialise serial port.
serial_port = 'COM3' # <---- Set name of serial port on this line.
ser = serial.Serial(serial_port, 38400, serial.EIGHTBITS, serial.PARITY_NONE, serial.STOPBITS_ONE, 1)

#Stay in a loop waiting for a connection request from the TI then processing that request, then wait
#for another connection request and process that request, and so on.
while True:

    #Wait for connection request from the TI. This will be in the form 'Cnnn.nnn.nnn.nnn/80[>0D]'.
    print ('Waiting for connection request from TI.')
    serial_text = read_from_serial_port()

    #Check that the text received from the serial port is connection request.
    if serial_text[0] == 'C' and serial_text[1:4].isdigit() and serial_text[4] == '.' and \
                                 serial_text[5:8].isdigit() and serial_text[8] == '.' and \
                                 serial_text[9:12].isdigit() and serial_text[12] == '.' and \
                                 serial_text[13:16].isdigit():

        #Isolate the IP address from the connection request. Then get rid of any leading zero in each octet. Although
        #they're not a problem with the UDS-10, a leading zero makes Windows interpet the octet as an octal value.
        ip_address = serial_text[1:16]

        if ip_address[0] == '0':
            ip_address = ip_address[1:]

        ip_address = ip_address.replace('.0', '.')

        print ('Received connection request to IP address: %s' % ip_address)

        #Open a socket connection to the specified IP address. It is assumed that each connection is to the web server
        #port 80.
        print ('Configuring socket to IP address.')
        my_sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        my_sock.connect((ip_address, 80))

        #Send a single character 'C' back to the TI to indicate a successful connection.
        print ('Sending connection response back to TI.')
        ser.write(b'C')

        #Wait for the HTTP GET request from the TI. This is in the following format, received as one string:
        #GET /<web page path and name> HTTP/1.0[>0D][>0A]
        #User-Agent: TI<browser_version_number>[>0D][>0A]
        #Host: <host name from keyboard buffer>[>0D][>0A][>0D][>0A]
        print ('Waiting for HTTP GET request from TI.')
        serial_text = read_from_serial_port()

        #Send HTTP GET request over the socket to the web server.
        print ('Sending HTTP GET request to server.')
        my_sock.send(serial_text.encode())

        #Get the web page returned over the socket from the web server in chunks of 1024 bytes
        #and send to the TI.
        print ('Getting web page from server and sending to TI.')
        while True:
            web_page = my_sock.recv(1024)
            if not web_page: break
            ser.write(web_page)

        #Finished processing this connection request. Close the socket.
        print ('Done.\n')               
        my_sock.close()
               
    else:
        #Text received from the serial port was not a connection request. Loop and wait for
        #the next text from the serial port.
        print ('Did not receive expected connect string from TI; received: %s' % serial_text)
